import 'package:bloc/bloc.dart';
import 'package:replay_cubit/replay_cubit.dart';

/// {@template replay_bloc}
/// A specialized [Bloc] which supports `undo` and `redo` operations.
///
/// [ReplayBloc] accepts an optional `limit` which determines
/// the max size of the history.
///
/// A custom [ReplayBloc] can be created by extending [ReplayBloc].
///
/// ```dart
/// class CounterBloc extends ReplayBloc<int> {
///   CounterBloc() : super(0);
///
///   void increment() => emit(state + 1);
/// }
/// ```
///
/// Then the built-in `undo` and `redo` operations can be used.
///
/// ```dart
/// final bloc = CounterBloc();
///
/// bloc.increment();
/// print(bloc.state); // 1
///
/// bloc.undo();
/// print(bloc.state); // 0
///
/// bloc.redo();
/// print(bloc.state); // 1
/// ```
///
/// The undo/redo history can be destroyed at any time by calling `clear`.
///
/// See also:
///
/// * [Bloc] for information about the [ReplayBloc] superclass.
///
/// {@endtemplate}
abstract class ReplayBloc<Event, State> extends Bloc<Event, State>
    with ReplayMixin<State> {
  /// {@macro replay_bloc}
  ReplayBloc(State state, {int limit}) : super(state) {
    this.limit = limit;
  }
}
